#ifndef ALLOWEDBASEPAIRS_H_
#define ALLOWEDBASEPAIRS_H_

#include <set>
#include "Alphabet.hh"

namespace ell
{
	/*! This class handels possible base pairs of an RNA structure	
	 *
	 * @author Martin Mann, Sebastian Will, Andreas Richter
	 */	
	class AllowedBasePairs {
	private:
			//! Allowed base pairs of an RNA structure
		typedef std::pair<Alphabet::AlphElem, Alphabet::AlphElem> BPair;
		
			//! Set of allowed base pairs of an RNA structure
		std::set< BPair > allowedBasePairSet;
		
			//! the Alphabet the allowed base pairs are basing on
		const Alphabet* const alph;
		
	public:
			/*! Construction 
			 * @param _alph has to be an Alphabet for a RNA
			 * @param bpSet the allowed combinations of alphabet elements
			 */
		AllowedBasePairs(const Alphabet* const _alph,
			const std::set< std::pair<std::string,std::string> >& bpSet);
		
		~AllowedBasePairs();

		bool operator== (const AllowedBasePairs& abp2) const;
		bool operator!= (const AllowedBasePairs& abp2) const;
		
			/*! Returns if two characters form a valid base pair, e.g.
			 *  a Watson-Crick (A-U, C-G) or a non-standard (G-U) pair.
			 */
		bool allowedBasePair(	const Alphabet::AlphElem& first,
			 	 				const Alphabet::AlphElem& second) const;
			 	 				
			 //! Returns the alphabet the base pairs are basing on
		const Alphabet* const getAlphabet() const { 
			return alph; 
		}
	};	

} // namespace ell

#endif /*ALLOWEDBASEPAIRS_H_*/
