#ifndef ENERGYFUNCTION_H_
#define ENERGYFUNCTION_H_


#include "Alphabet.hh"
#include <biu/Matrix.hh>



namespace ell
{

		/*! The contact energy matrix for a ContactEnergyFunction object. */
	typedef biu::Matrix<double> EnergyMatrix;


		/*! This class manages tasks on contact energy functions.
		 * 
		 * It provides the contact energy for elements of a specified alphabet.
		 * 
		 * @author Martin Mann, Sebastian Will, Andreas Richter
		 */
	class ContactEnergyFunction
	{
	private:
	
		const Alphabet* const alphabet;
		const EnergyMatrix* const energyMat;

	public:
	
			/*! The alphabet elements of type AlphEleme are mapped to
			 * the EnergyMatrix indices via the getIndex(..) of the
			 * Alphabet class.
			 */
		ContactEnergyFunction(	const Alphabet* const _alphabet, 
								const EnergyMatrix* const _energyMat);
								
		virtual ~ContactEnergyFunction();
			
			/*! Returns the contact energy of two elements of the alphabet. 
			 * 
			 * @param first,second have to be elements of the specified alphabet.*/
		double getContactEnergy(	const Alphabet::AlphElem& first, 
									const Alphabet::AlphElem& second) const;
									
			/*! Returns the alphabet this energy function is based on. */
		const Alphabet* const getAlphabet() const { 
			return alphabet; 
		}
		
		bool operator == (const ContactEnergyFunction& cef2) const;
		bool operator != (const ContactEnergyFunction& cef2) const;
		
	}; // class

} // namespace ell

#endif /*ENERGYFUNCTION_H_*/
