#ifndef LATTICE_NEIGHBORHOOD_HH_
#define LATTICE_NEIGHBORHOOD_HH_


#include "NeighborVector.hh"

namespace ell
{
		/*! 
		 * A LatticeNeighborhood handles the
		 * the NeighborVector objects of a lattice and the 
		 * access to them.
		 */
	class LatticeNeighborhood
	{
	private:
		const MoveAlphabet* moveAlph;	//!< the underlying move string alphabet
		NeighSet			neighSet;	//!< the NeighborVector data
		
			//! a crossreference from move index to NeighborVectors
		std::vector<const NeighborVector*>	neighVec;	
		
	public:
		LatticeNeighborhood(	const MoveAlphabet* moveAlph_, 
								const NeighSet& neighbors);
								
		virtual ~LatticeNeighborhood();
		
			//! Returns the number of elements of the neighborhood.
		unsigned int size() const;
		
			//! Returns whether or not a vector belongs to the neighborhood.
		bool isElement( const IntPoint& vector) const;
		
			//! Returns the corresponding NeighborVector.
			//! @param vector has to be element of this neighborhood
		const NeighborVector& getElement(const IntPoint& vector) const;
		
			//! Returns the corresponding NeighborVector.
			//! @param move has to be element of the underlying move string 
			//!				alphabet
		const NeighborVector& getElement(const Move& move) const;
		
			//! a constant iterator to access the elements of the neighborhood
		typedef NeighSet::const_iterator const_iterator;
		
			//! Returns a constant iterator to the first element of the 
			//! neighborhood
		const_iterator begin() const;
		
			//! Returns a constant iterator behind the last element of the 
			//! neighborhood
		const_iterator end() const;
	};

} // namespace ell

#endif /*LATTICE_NEIGHBORHOOD_HH_*/
