#include <ell/RNAFreeEnergy.hh>
#include <ell/assertell.hh>


extern "C"	float energy_of_struct(const char *string, const char *structure);

namespace ell
{

	RNAFreeEnergy::RNAFreeEnergy(	const std::string& rnaSeqStr,
	 								const Structure& _rnaStructBracketNot,
	 								const AllowedBasePairs* const _bPair)
	 :	RNAStructure(rnaSeqStr, _rnaStructBracketNot, _bPair) 
	{
	}
	
	RNAFreeEnergy::RNAFreeEnergy(const RNAFreeEnergy& rnaFreeEnerg)
	 : RNAStructure(rnaFreeEnerg) 
	{
	}

	RNAFreeEnergy::~RNAFreeEnergy()	{
	}

	unsigned int	
	RNAFreeEnergy::nextCompatibleSingleMovePos( unsigned int pos) const {
		if (pos+1 < rnaStructBracketNot->size()) {
			switch ((*rnaStructBracketNot)[pos+1]) {
				case '.' : return pos+1; break;
				case ')' : return RNAStructure::INVALID_INDEX;
					   break;
				case '(' : return nextCompatibleSingleMovePos(
					   	   rnaBonds[pos+1]);
					   break;
				}
			return RNAStructure::INVALID_INDEX;
		}
		else {
			return RNAStructure::INVALID_INDEX;
		}
	}

	void		
	RNAFreeEnergy::applySingleMove(const SingleMove& move) {
		// open bond between move.first and move.second
		if (rnaBonds[move.first] == move.second) {
			(*rnaStructBracketNot)[move.first] = '.';
			(*rnaStructBracketNot)[move.second] = '.';
			rnaBonds[move.first] = RNAStructure::INVALID_INDEX;
		}
		// close bond between pos1 and pos2
		else {
			assertell(isValidSingleMove(move),
				"cant close bond with single move, because it is not valid");
			(*rnaStructBracketNot)[move.first] = '(';
			(*rnaStructBracketNot)[move.second] = ')';
			rnaBonds[move.first] = move.second;
		}
	}

	bool		
	RNAFreeEnergy::isValidSingleMove(const SingleMove& move)
			 const {
		if (move.first >= move.second
		    || move.second >= rnaStructBracketNot->size()) {
			return false;
		}

		// bond between move.first and move.second
		if (rnaBonds[move.first] == move.second) {
			return true;
		}
		// no bond between move.first and move.second
		// move.first or move.second are paired, but not with each other
		if (	!((*rnaStructBracketNot)[move.first] == '.' 
				&& (*rnaStructBracketNot)[move.second] == '.')) {
			return false;
		}

		// both move.first and move.second are unpaired
		// distance between move.first and move.second has to be at least
		//  MIN_LOOP_LENGTH
		if ((move.second-move.first) <= MIN_LOOP_LENGTH) {
			return false;
		}
		// iterate all positions which preserve the nested property
		// starting after move.first to determine if move.second is one
		// of this positions
		for (unsigned int i = nextCompatibleSingleMovePos(move.first);
		     i < rnaStructBracketNot->size() && i <= move.second;
		     i = nextCompatibleSingleMovePos(i)) {
			if (move.second == i) {
				return isAllowedBasePair(move.first, move.second);
			}
		}
		// bond between move.first and move.second would
		// destroy the nested property
		return false;
	}
	
	double 		
	RNAFreeEnergy::getEnergy() const {
		return energy_of_struct(
			(bPair->getAlphabet()->getString(*rnaSeq)).c_str(),
			rnaStructBracketNot->c_str());
	}

} // namespace ell
